"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Retrycf = require("../retrycf");
const Helper = require("./firebaseHelper");
const Model = require("./sampleModel");
require("jest");
jest.setTimeout(20000);
beforeAll(() => {
    const _ = Helper.Firebase.shared;
});
const step = 'step';
describe('clearCompleted', () => {
    test('clear', () => __awaiter(this, void 0, void 0, function* () {
        let order = new Model.SampleOrder();
        const neoTask = new Retrycf.NeoTask();
        const completed = { [step]: true };
        neoTask.completed = completed;
        order.neoTask = neoTask.rawValue();
        yield order.save();
        expect(order.neoTask.completed[step]).toEqual(true);
        // chack callback order
        order = yield Retrycf.NeoTask.clearCompleted(order);
        expect(order.neoTask.completed).toBeUndefined();
        const fetchedOrder = yield Model.SampleOrder.get(order.id);
        expect(fetchedOrder.neoTask.completed).toBeUndefined();
    }));
});
describe('isCompleted', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
    }));
    describe('when neoTask is undefined', () => {
        test('false', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            expect(Retrycf.NeoTask.isCompleted(order, 'step')).toEqual(false);
        }));
    });
    describe('when step is not completed', () => {
        test('false', () => __awaiter(this, void 0, void 0, function* () {
            const completed = { 'other': true };
            const neoTask = new Retrycf.NeoTask();
            neoTask.completed = completed;
            order.neoTask = neoTask.rawValue();
            expect(Retrycf.NeoTask.isCompleted(order, 'step')).toEqual(false);
        }));
    });
    describe('when step is completed', () => {
        test('true', () => __awaiter(this, void 0, void 0, function* () {
            const completed = { [step]: true };
            const neoTask = new Retrycf.NeoTask();
            neoTask.completed = completed;
            order.neoTask = neoTask.rawValue();
            expect(Retrycf.NeoTask.isCompleted(order, 'step')).toEqual(true);
        }));
    });
}));
describe('setRetry', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
        yield order.save();
    }));
    describe('when retry count is undefined', () => {
        test('retry added and count is 1', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            const updatedOrder = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            expect(updatedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
            expect(updatedOrder.neoTask.retry.error).toEqual(['error']);
            expect(updatedOrder.neoTask.retry.count).toEqual(1);
            const fetchedOrder = yield Model.SampleOrder.get(order.id);
            expect(fetchedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
            expect(fetchedOrder.neoTask.retry.error).toEqual(['error']);
            expect(fetchedOrder.neoTask.retry.count).toEqual(1);
            // check Failure
            const failures = yield Retrycf.Failure.querySnapshot(fetchedOrder.reference.path);
            expect(failures.docs.length).toEqual(1);
            const failure = new Retrycf.Failure();
            failure.init(failures.docs[0]);
            expect(failure.refPath).toEqual(fetchedOrder.reference.path);
            expect(failure.neoTask.status).toEqual(fetchedOrder.neoTask.status);
        }));
    });
    describe('when retry count is already exist', () => {
        test('count is 2', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            const updated1Order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error1');
            const updated2Order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error2');
            expect(updated2Order.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
            expect(updated2Order.neoTask.retry.error).toEqual(['error1', 'error2']);
            expect(updated2Order.neoTask.retry.count).toEqual(2);
            const fetchedOrder = yield Model.SampleOrder.get(order.id);
            expect(fetchedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
            expect(fetchedOrder.neoTask.retry.error).toEqual(['error1', 'error2']);
            expect(fetchedOrder.neoTask.retry.count).toEqual(2);
            // check Failure
            const failures = yield Retrycf.Failure.querySnapshot(fetchedOrder.reference.path);
            expect(failures.docs.length).toEqual(1);
            const failure = new Retrycf.Failure();
            failure.init(failures.docs[0]);
            expect(failure.refPath).toEqual(fetchedOrder.reference.path);
            expect(failure.neoTask.status).toEqual(fetchedOrder.neoTask.status);
        }));
    });
}));
describe('setInvalid', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
        yield order.save();
    }));
    test('set invalid', () => __awaiter(this, void 0, void 0, function* () {
        order.neoTask = undefined;
        const validationError = new Retrycf.ValidationError('errortype', 'reason');
        const updatedOrder = yield Retrycf.NeoTask.setInvalid(order, validationError);
        expect(updatedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
        expect(updatedOrder.neoTask.invalid.validationError).toEqual('errortype');
        expect(updatedOrder.neoTask.invalid.reason).toEqual('reason');
        const fetchedOrder = yield Model.SampleOrder.get(order.id);
        expect(updatedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
        expect(updatedOrder.neoTask.invalid.validationError).toEqual('errortype');
        expect(updatedOrder.neoTask.invalid.reason).toEqual('reason');
    }));
}));
describe('setFatal', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
        yield order.save();
    }));
    test('set fatal', () => __awaiter(this, void 0, void 0, function* () {
        order.neoTask = undefined;
        const updatedOrder = yield Retrycf.NeoTask.setFatal(order, 'step', 'error');
        expect(updatedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
        expect(updatedOrder.neoTask.fatal.error).toEqual('error');
        expect(updatedOrder.neoTask.fatal.step).toEqual('step');
        const fetchedOrder = yield Model.SampleOrder.get(order.id);
        expect(fetchedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
        expect(fetchedOrder.neoTask.fatal.error).toEqual('error');
        expect(fetchedOrder.neoTask.fatal.step).toEqual('step');
        // check Failure
        const failures = yield Retrycf.Failure.querySnapshot(fetchedOrder.reference.path);
        expect(failures.docs.length).toEqual(1);
        const failure = new Retrycf.Failure();
        failure.init(failures.docs[0]);
        expect(failure.refPath).toEqual(fetchedOrder.reference.path);
        expect(failure.neoTask.status).toEqual(fetchedOrder.neoTask.status);
    }));
}));
describe('setFatal', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
        yield order.save();
    }));
    test('set fatal', () => __awaiter(this, void 0, void 0, function* () {
        order.neoTask = undefined;
        const updatedOrder = yield Retrycf.NeoTask.setFatal(order, 'step', 'error');
        expect(updatedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
        expect(updatedOrder.neoTask.fatal.error).toEqual('error');
        expect(updatedOrder.neoTask.fatal.step).toEqual('step');
        const fetchedOrder = yield Model.SampleOrder.get(order.id);
        expect(fetchedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
        expect(fetchedOrder.neoTask.fatal.error).toEqual('error');
        expect(fetchedOrder.neoTask.fatal.step).toEqual('step');
        // check Failure
        const failures = yield Retrycf.Failure.querySnapshot(fetchedOrder.reference.path);
        expect(failures.docs.length).toEqual(1);
        const failure = new Retrycf.Failure();
        failure.init(failures.docs[0]);
        expect(failure.refPath).toEqual(fetchedOrder.reference.path);
        expect(failure.neoTask.status).toEqual(fetchedOrder.neoTask.status);
    }));
}));
describe('getRetryCount', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
        yield order.save();
    }));
    describe('when neotask is undefined', () => {
        test('undefined', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            expect(Retrycf.NeoTask.getRetryCount(order)).toBeUndefined();
        }));
    });
    describe('when retry set', () => {
        test('one', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            const updatedOrder = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            expect(Retrycf.NeoTask.getRetryCount(order)).toEqual(1);
        }));
    });
}));
describe('getRetryCount', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    let preOrder;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
        preOrder = new Model.SampleOrder();
        yield order.save();
        yield preOrder.save();
    }));
    describe('current order retry count is undefined', () => {
        test('false', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            expect(Retrycf.NeoTask.shouldRetry(order, undefined)).toEqual(false);
        }));
    });
    describe('current order retry count is over 3', () => {
        test('false', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            expect(Retrycf.NeoTask.shouldRetry(order, undefined)).toEqual(false);
        }));
    });
    describe('current order retry count is under 2 and previous count is undefined', () => {
        test('true', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            expect(Retrycf.NeoTask.shouldRetry(order, undefined)).toEqual(true);
        }));
    });
    describe('current order retry count is 2 and previous count is 1', () => {
        test('true', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            preOrder.neoTask = undefined;
            preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
            expect(Retrycf.NeoTask.shouldRetry(order, preOrder)).toEqual(true);
        }));
    });
    describe('current order retry count is 1 and previous count is 2', () => {
        test('false', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            preOrder.neoTask = undefined;
            preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
            preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
            expect(Retrycf.NeoTask.shouldRetry(order, preOrder)).toEqual(false);
        }));
    });
}));
describe('setFatalIfRetryCountIsMax', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    let preOrder;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
        preOrder = new Model.SampleOrder();
        yield order.save();
        yield preOrder.save();
    }));
    describe('current retry count is undefined', () => {
        test('failure not set', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            yield Retrycf.NeoTask.setFatalIfRetryCountIsMax(order, undefined);
            const failures = yield Retrycf.Failure.querySnapshot(order.reference.path);
            expect(failures.docs.length).toEqual(0);
        }));
    });
    describe('current and previous retry count is undefined', () => {
        test('failure not set', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            preOrder.neoTask = undefined;
            yield Retrycf.NeoTask.setFatalIfRetryCountIsMax(order, preOrder);
            const failures = yield Retrycf.Failure.querySnapshot(order.reference.path);
            expect(failures.docs.length).toEqual(0);
        }));
    });
    describe('only previous retry count is undefined', () => {
        test('fatal not set', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
            preOrder.neoTask = undefined;
            yield Retrycf.NeoTask.setFatalIfRetryCountIsMax(order, preOrder);
            const failures = yield Retrycf.Failure.querySnapshot(order.reference.path);
            const failure = new Retrycf.Failure();
            failure.init(failures.docs[0]);
            expect(failure.neoTask.fatal).toBeUndefined();
        }));
    });
    describe('both current and previous are not undefined', () => {
        describe('current retry count is over MAX', () => {
            describe('current retry count > prevous retry count', () => {
                test('fatal set', () => __awaiter(this, void 0, void 0, function* () {
                    order.neoTask = undefined;
                    order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
                    order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
                    order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
                    order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
                    preOrder.neoTask = undefined;
                    preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
                    preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
                    preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
                    order = yield Retrycf.NeoTask.setFatalIfRetryCountIsMax(order, preOrder);
                    expect(order.neoTask.fatal.step).toEqual('retry_failed');
                    expect(order.neoTask.fatal.error).toEqual('retry failed');
                    const failures = yield Retrycf.Failure.querySnapshot(order.reference.path);
                    const failure = new Retrycf.Failure();
                    failure.init(failures.docs[0]);
                    expect(failure.neoTask.fatal.step).toEqual('retry_failed');
                    expect(failure.neoTask.fatal.error).toEqual('retry failed');
                }));
            });
            describe('current retry count < prevous retry count', () => {
                test('fatal not set', () => __awaiter(this, void 0, void 0, function* () {
                    order.neoTask = undefined;
                    order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
                    preOrder.neoTask = undefined;
                    preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
                    preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
                    yield Retrycf.NeoTask.setFatalIfRetryCountIsMax(order, preOrder);
                    const failures = yield Retrycf.Failure.querySnapshot(order.reference.path);
                    const failure = new Retrycf.Failure();
                    failure.init(failures.docs[0]);
                    expect(failure.neoTask.fatal).toBeUndefined();
                }));
            });
        });
        describe('current retry count is under MAX', () => {
            test('fatal not set', () => __awaiter(this, void 0, void 0, function* () {
                order.neoTask = undefined;
                order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
                order = yield Retrycf.NeoTask.setRetry(order, 'step', 'error');
                preOrder.neoTask = undefined;
                preOrder = yield Retrycf.NeoTask.setRetry(preOrder, 'step', 'error');
                yield Retrycf.NeoTask.setFatalIfRetryCountIsMax(order, preOrder);
                const failures = yield Retrycf.Failure.querySnapshot(order.reference.path);
                const failure = new Retrycf.Failure();
                failure.init(failures.docs[0]);
                expect(failure.neoTask.fatal).toBeUndefined();
            }));
        });
    });
}));
describe('setSuccess', () => __awaiter(this, void 0, void 0, function* () {
    let order;
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        order = new Model.SampleOrder();
        yield order.save();
    }));
    describe('failure exists', () => {
        test('failure deleted', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            order = yield Retrycf.NeoTask.setFatal(order, 'step', 'error');
            expect(order.neoTask.status).toEqual(Retrycf.NeoTaskStatus.failure);
            let failures = yield Retrycf.Failure.querySnapshot(order.reference.path);
            expect(failures.docs.length).toEqual(1);
            order = yield Retrycf.NeoTask.setSuccess(order);
            expect(order.neoTask.status).toEqual(Retrycf.NeoTaskStatus.success);
            const fetchedOrder = yield Model.SampleOrder.get(order.id);
            expect(fetchedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.success);
            // check Failure deleted
            failures = yield Retrycf.Failure.querySnapshot(order.reference.path);
            expect(failures.docs.length).toEqual(0);
        }));
    });
    describe('failure not exists', () => {
        test('success', () => __awaiter(this, void 0, void 0, function* () {
            order.neoTask = undefined;
            order = yield Retrycf.NeoTask.setSuccess(order);
            expect(order.neoTask.status).toEqual(Retrycf.NeoTaskStatus.success);
            const fetchedOrder = yield Model.SampleOrder.get(order.id);
            expect(fetchedOrder.neoTask.status).toEqual(Retrycf.NeoTaskStatus.success);
        }));
    });
}));
